/* Program version */
#define VERSION "4.4.13"

/* Configurable features */

/* 6bone referto: support */
#define EXT_6BONE

/* Suppress lame NSI disclaimer */
#define HIDE_DISCL

/* Always ask INTERNIC about gTLDs */
#define FIRST_ASK_INTERNIC

/* Default server */
#define DEFAULTSERVER   "whois.internic.net"

